// for audio record
let audio_sample_rate = null;
let scriptProcessor = null;
let audioContext = null;
let mediastreamsource = null;
let bufferSize = 2048;

// for tone analyze
const Ntone = 24;
const tone_lower_boundary = 105;
const tone_upper_boundary = [113, 120, 127, 135, 142, 150, 160, 170, 180, 190, 202, 215,
			     226, 240, 254, 269, 285, 302, 320, 339, 369, 380, 402, 428];
const freq_name = ['A', 'Bb', 'B', 'C', 'C#', 'D', 'Eb', 'E', 'F', 'F#', 'G', 'Ab',
		   'A', 'Bb', 'B', 'C', 'C#', 'D', 'Eb', 'E', 'F', 'F#', 'G', 'Ab'];

function init_microphone(){
  //様々なブラウザでマイクへのアクセス権を取得する
  navigator.getUserMedia = navigator.getUserMedia || navigator.webkitGetUserMedia;
  //audioのみtrue。Web Audio APIが問題なく使えるのであれば、第二引数で指定した関数を実行する
  navigator.getUserMedia({audio: true, video: false}, successFunc, errorFunc);
}


function successFunc(stream) {
    audioContext = new AudioContext();
    audio_sample_rate = audioContext.sampleRate;
    console.log(audio_sample_rate);
    scriptProcessor = audioContext.createScriptProcessor(bufferSize, 1, 1);
    mediastreamsource = audioContext.createMediaStreamSource(stream);
    interval = bufferSize / audio_sample_rate;
}

// Web Audio APIが使えなかった時
function errorFunc(error) {
    alert("error: Web Audio API is not available.");
}

function startRecord() {
    mediastreamsource.connect(scriptProcessor);
    scriptProcessor.onaudioprocess = onAudioProcess;
    scriptProcessor.connect(audioContext.destination);
}

// save audio data
var onAudioProcess = function (e) {
    var volume = 0.0;
    var input = e.inputBuffer.getChannelData(0);
    var bufferData = new Float32Array(bufferSize);
    for (var i = 0; i < bufferSize; i++) {
        bufferData[i] = input[i];
	volume += Math.abs(input[i]);
    }
    volume = (volume / bufferSize) * 32000;
    //console.log(volume)
    var tone = getTone(bufferData, audio_sample_rate);
    showResult(tone, volume)
};


function stopRecord(){
    scriptProcessor.disconnect();
    mediastreamsource.disconnect();
}


function getTone(data){
    var length = data.length;
    var sc = new Array(length);
    var self = new Array(length * 2);
    var scl = Math.floor(audio_sample_rate / tone_upper_boundary[Ntone - 1]);
    var sch = Math.ceil(audio_sample_rate / tone_lower_boundary);
    var max = 0.0;
    var n = 0;

    for(var i = 0; i < length; i++){
	self[i] = data[i];
	self[length + i] = data[i];
    }

    for(var i = scl; i < sch; i++){
	sc[i] = 0.0;
	for(var j = 0; j < length; j++){
	    sc[i] += (self[i+j] * data[j]);
	}
    }

    for(var i = scl; i < sch; i++){
	if(sc[i] > max){
	    max = sc[i];
	    n = i;
	}
    }

    var f0 = audio_sample_rate / n;
    var tone = tonename(f0);
    return tone;
}

function tonename(f0){

    if(f0 < tone_lower_boundary){
	return freq_name[Ntone - 1];
    }

    for(var i = 0; i < Ntone; i++){
	if(f0 < tone_upper_boundary[i]){
	    return freq_name[i];
	}
    }
    return freq_name[0];

}






